 
/*
 * gvfTest.cc
 *
 *  Created on: 13-Feb-2009
 *      Author: asingh
 */

//ITK Headers
#include "itkImage.h"

#include "itkBinaryThresholdImageFilter.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkCovariantVector.h"
#include "itkGradientVectorFlowImageFilter.h"
#include "itkGradientMagnitudeRecursiveGaussianImageFilter.h"
#include "itkGradientMagnitudeImageFilter.h"
#include "itkGradientImageFilter.h"
#include "itkCastImageFilter.h"
#include "itkImageToVTKImageFilter.h"
#include "itkImageDuplicator.h"
#include "itkImageIORegion.h"
#include "itkImageRegionIterator.h"
#include "itkGeodesicActiveContourLevelSetImageFilter.h"
#include "itkTPGACLevelSetImageFilter_18_6.h"
#include "itkRecursiveGaussianImageFilter.h"

#include <cmath>


int main( int argc, char *argv[] )
{

	if (argc < 8)
	{
		std::cerr << "Usage: gvfTest gvfNoiseLevel gvf#iterations prop_sc curv_sc adv_sc rms_error lvlset#iterations" << endl;
		return 1;
	}

    typedef   float  InternalPixelType;
    typedef float OutputPixelType;
    typedef unsigned char BinaryPixelType;

    const     unsigned int    Dimension = 2;

    typedef itk::Image< InternalPixelType, Dimension >  InternalImageType;
    typedef itk::Image< OutputPixelType, Dimension > OutputImageType;
    typedef itk::Image< BinaryPixelType, Dimension > BinaryImageType;

    typedef  itk::ImageFileWriter<  InternalImageType  > WriterType;
    typedef  itk::ImageFileWriter<  BinaryImageType  > BinaryWriterType;
    typedef  itk::ImageFileReader<  BinaryImageType  > BinaryReaderType;
    typedef  itk::ImageFileReader<  InternalImageType  > ReaderType;

    typedef itk::ImageRegionIterator< InternalImageType > IteratorType;
    typedef itk::ImageRegionIterator< BinaryImageType > BinaryIteratorType;
    typedef itk::ImageRegionConstIterator< InternalImageType > ConstIteratorType;

    typedef itk::GeodesicActiveContourLevelSetImageFilter<InternalImageType, InternalImageType> GeodesicActiveContourFilterType;
   	  GeodesicActiveContourFilterType::Pointer geodesicActiveContour = GeodesicActiveContourFilterType::New();

    typedef itk::BinaryThresholdImageFilter< InternalImageType, OutputImageType > ThresholdingFilterType;
  ThresholdingFilterType::Pointer thresholder = ThresholdingFilterType::New();

  thresholder->SetLowerThreshold( -1000 );
  thresholder->SetUpperThreshold( 0  );

  thresholder->SetOutsideValue(  0  );
  thresholder->SetInsideValue(  255 );

  //save image

  ReaderType::Pointer readerRing = ReaderType::New();

   //save image
  readerRing->SetFileName("./gvfTest3.nii");

  try
  {
	  readerRing->Update();
  }
  catch( itk::ExceptionObject & excep )
  {
	 std::cerr << "Exception caught !" << std::endl;
	 std::cerr << excep << std::endl;
  }

  	//***************************************GRADIENT VECTOR FLOW***************************************************
	typedef itk::CovariantVector< float, Dimension >  GradientPixelType;
	typedef itk::Image< GradientPixelType, Dimension > GradientImageType;
	typedef itk::GradientVectorFlowImageFilter< GradientImageType, GradientImageType > GVFFilterType;
	//typedef itk::LaplacianImageFilter<InternalImageType, InternalImageType> LaplacianFilterType;
	typedef itk::GradientImageFilter<InternalImageType, float, float> GradientImageFilterType;
	typedef GeodesicActiveContourFilterType::VectorImageType AdvectionImageType;
	typedef itk::CastImageFilter<GradientImageType,AdvectionImageType> gvfCastFilterType;
	typedef itk::ImageToVTKImageFilter<InternalImageType> ConnectorType;
  	typedef itk::GradientMagnitudeImageFilter<InternalImageType, InternalImageType> GradientMagnitudeImageFilterType;
  	typedef itk::GradientMagnitudeRecursiveGaussianImageFilter<InternalImageType, InternalImageType> GradientMagnitudeRecursiveGaussianImageFilterType;
  	typedef itk::RecursiveGaussianImageFilter<InternalImageType, InternalImageType> RecursiveGaussianFilterType;

	GradientImageFilterType::Pointer gFilter = GradientImageFilterType::New();
	//gFilter->SetUseImageDirection(false);
	gFilter->SetInput(readerRing->GetOutput());

	try
	{
		gFilter->Update();
	}
	catch( itk::ExceptionObject & excep )
	{
	  std::cerr << "Exception caught !" << std::endl;
	  std::cerr << excep << std::endl;
	}

	//  Next we construct one filter of this class using the \code{New()}
	//  method.
	//
	GVFFilterType::Pointer gvfFilter = GVFFilterType::New();
	gvfFilter->SetInput(gFilter->GetOutput());

	//DEBUG///////////////////////////////////////////////////////////////////////////
// 	WriterType::Pointer gmWriter = WriterType::New();
// 	gmWriter->SetInput(gmFilter->GetOutput());
// 	gmWriter->SetFileName("./gradientMagnitude3.nii");
// 	gmWriter->Update();

	typedef itk::ImageFileWriter< GradientImageType > FieldWriterType;
	FieldWriterType::Pointer gradientWriter = FieldWriterType::New();
	gradientWriter->SetInput(gFilter->GetOutput());
	gradientWriter->SetFileName("./gradientRing3.vtk");
	try
	{
		gradientWriter->Update();
	}
	catch( itk::ExceptionObject & excep )
	{
	  std::cerr << "Exception caught !" << std::endl;
	  std::cerr << excep << std::endl;
	}
	/////////////////////////////////////////////////////////////////////////////////

	gvfCastFilterType::Pointer castFilter = gvfCastFilterType::New();


	//LaplacianFilterType::Pointer LFilter = LaplacianFilterType::New();
	//gvfFilter->SetLaplacianFilter(LFilter);
	gvfFilter->SetNoiseLevel(atof(argv[1]));
	gvfFilter->SetIterationNum( atoi(argv[2]));

	typedef itk::ImageFileWriter< AdvectionImageType > FieldWriterType2;
	argv[9] = "./gvfRing3.vtk";
	FieldWriterType2::Pointer fieldWriter = FieldWriterType2::New();
	fieldWriter->SetFileName( argv[9] );
	castFilter->SetInput(gvfFilter->GetOutput());
	fieldWriter->SetInput( castFilter->GetOutput() );

	try
	{
		fieldWriter->Update();
	}
	catch( itk::ExceptionObject & excep )
	{
	  std::cerr << "Exception caught !" << std::endl;
	  std::cerr << excep << std::endl;
	}


  //***************************************GRADIENT VECTOR FLOW END***************************************************



		  ReaderType::Pointer speedReader = ReaderType::New();
		  speedReader->SetFileName( "./speedImage2.nii" );

		  try
		  {
			speedReader->Update();
		  }
		  catch( itk::ExceptionObject & excep )
		  {
			std::cerr << "Exception caught !" << std::endl;
			std::cerr << excep << std::endl;
		  }

	argv[8] = "./gvfTestDistMap2.nii";
	  ReaderType::Pointer initialLevelSet = ReaderType::New();
	  initialLevelSet->SetFileName( argv[8] );

	  thresholder->SetInput( geodesicActiveContour->GetOutput() );

	  WriterType::Pointer writerLevelSet = WriterType::New();
	  WriterType::Pointer writerSurface = WriterType::New();

	  writerLevelSet -> SetFileName( "./gvfTestEvolvedLevelSet.nii" );
	  writerLevelSet -> SetInput( geodesicActiveContour->GetOutput() );

	  writerSurface -> SetFileName( "./gvfTestEvolved.nii" );
	  writerSurface -> SetInput( thresholder->GetOutput() );

	  //connect pipeline
	  geodesicActiveContour->SetInput(initialLevelSet->GetOutput());
	  geodesicActiveContour->SetFeatureImage(speedReader->GetOutput());
	  geodesicActiveContour->SetAdvectionImage(castFilter->GetOutput());

	  geodesicActiveContour -> SetPropagationScaling( atof(argv[3]) );
	  geodesicActiveContour -> SetCurvatureScaling( atof(argv[4]) );
	  geodesicActiveContour -> SetAdvectionScaling( atof(argv[5]) );

	  geodesicActiveContour -> SetMaximumRMSError( atof(argv[6]) );
	  geodesicActiveContour -> SetNumberOfIterations( atoi(argv[7]) );

	  try
	  {
		 writerLevelSet->Update();
		 writerSurface->Update();
	  }
	  catch( itk::ExceptionObject & excep )
	  {
		  std::cerr << "Exception caught !" << std::endl;
		  std::cerr << excep << std::endl;
	  }

	  // Print out some useful information
	  std::cout << std::endl;
	  std::cout << "Max. no. iterations: " << geodesicActiveContour->GetNumberOfIterations() << std::endl;
	  std::cout << "Max. RMS error: " << geodesicActiveContour->GetMaximumRMSError() << std::endl;
	  std::cout << std::endl;
	  std::cout << "No. elapsed iterations: " << geodesicActiveContour->GetElapsedIterations() << std::endl;
	  std::cout << "RMS change: " << geodesicActiveContour->GetRMSChange() << std::endl;

	return 0;


}
